@@ -11,7 +11,10 @@ class ProtocolCode(object):
     FOOTER = 0xFA
 
     # System status
-    VERSION = 0x00
+    ROBOT_VERSION = 0x01
+    SOFTWARE_VERSION = 0x02
+    GET_ROBOT_ID = 0x03
+    SET_ROBOT_ID = 0x04
 
     # Overall status
     POWER_ON = 0x10
@@ -20,6 +23,7 @@ class ProtocolCode(object):
     RELEASE_ALL_SERVOS = 0x13
     IS_CONTROLLER_CONNECTED = 0x14
     READ_NEXT_ERROR = 0x15
+    SET_FRESH_MODE = 0x16
     SET_FREE_MODE = 0x1A
     IS_FREE_MODE = 0x1B
 
@@ -39,20 +43,26 @@ class ProtocolCode(object):
 
     # JOG MODE AND OPERATION
     JOG_ANGLE = 0x30
+    JOG_ABSOLUTE = 0x31
     JOG_COORD = 0x32
+    JOG_INCREMENT = 0x33
     JOG_STOP = 0x34
     SET_ENCODER = 0x3A
     GET_ENCODER = 0x3B
     SET_ENCODERS = 0x3C
     GET_ENCODERS = 0x3D
+    SET_ENCODERS_DRAG = 0x3E
 
     # RUNNING STATUS AND SETTINGS
     GET_SPEED = 0x40
     SET_SPEED = 0x41
     GET_FEED_OVERRIDE = 0x42
     GET_ACCELERATION = 0x44
+    SET_ACCELERATION = 0x45
     GET_JOINT_MIN_ANGLE = 0x4A
     GET_JOINT_MAX_ANGLE = 0x4B
+    SET_JOINT_MIN = 0x4C
+    SET_JOINT_MAX = 0x4D
 
     # SERVO CONTROL
     IS_SERVO_ENABLE = 0x50
@@ -60,6 +70,7 @@ class ProtocolCode(object):
     SET_SERVO_DATA = 0x52
     GET_SERVO_DATA = 0x53
     SET_SERVO_CALIBRATION = 0x54
+    JOINT_BRAKE = 0x55
     RELEASE_SERVO = 0x56
     FOCUS_SERVO = 0x57
 
@@ -72,13 +83,17 @@ class ProtocolCode(object):
     GET_GRIPPER_VALUE = 0x65
     SET_GRIPPER_STATE = 0x66
     SET_GRIPPER_VALUE = 0x67
-    SET_GRIPPER_INI = 0x68
+    SET_GRIPPER_CALIBRATION = 0x68 
     IS_GRIPPER_MOVING = 0x69
     SET_COLOR = 0x6A
+    SET_ELETRIC_GRIPPER = 0x6B
+    INIT_ELETRIC_GRIPPER = 0x6C
 
     # Basic
     SET_BASIC_OUTPUT = 0xA0
     GET_BASIC_INPUT = 0xA1
+    GET_BASE_INPUT = 0xA2
+    SET_BASE_PWM = 0xA5
 
     # Linux GPIO, mode: GPIO.BCM
     SET_GPIO_MODE = 0xAA
@@ -93,7 +108,57 @@ class ProtocolCode(object):
 
     # Get the measured distance
     GET_TOF_DISTANCE = 0xC0
-
+    
+    # Coordinate transformation
+    SET_TOOL_REFERENCE = 0x81
+    GET_TOOL_REFERENCE = 0x82
+    SET_WORLD_REFERENCE = 0x83
+    GET_WORLD_REFERENCE = 0x84
+    SET_REFERENCE_FRAME = 0x85
+    GET_REFERENCE_FRAME = 0x86
+    SET_MOVEMENT_TYPE = 0x87
+    GET_MOVEMENT_TYPE = 0x88
+    SET_END_TYPE = 0x89
+    GET_END_TYPE = 0x8A
+    
+    # Impact checking
+    SET_JOINT_CURRENT = 0x90
+    GET_JOINT_CURRENT = 0x91
+    SET_CURRENT_STATE = 0x92
+    
+    # planning speed
+    GET_PLAN_SPEED = 0xD0
+    GET_PLAN_ACCELERATION = 0xD1
+    SET_PLAN_SPEED = 0xD2
+    SET_PLAN_ACCELERATION = 0xD3
+    
+    # Motor status read
+    GET_SERVO_SPEED = 0xE1
+    GET_SERVO_CURRENTS = 0xE2
+    GET_SERVO_VOLTAGES = 0xE3
+    GET_SERVO_STATUS = 0xE4
+    GET_SERVO_TEMPS = 0xE5
+    
+    # IIC
+    # SET_IIC_STATE = 0xA4
+    # GET_IIS_BYTE = 0xA5
+    # SET_IIC_BYTE = 0xA6 
+    
+    # mypalletizer lite
+    END = "\r"
+    COORDS_SET = "g0"
+    BACK_ZERO = "g28"
+    SLEEP_TIME = "g4"
+    ABS_CARTESIAN = "g90"
+    REL_CARTESIAN = "g91"
+    SET_JOINT = "g92"
+    GIRPPER_OPEN = "M3"
+    GIRPPER_CLOSE = "M4"
+    RELEASE_SERVOS = "M17"
+    LOCK_SERVOS = "M18"
+    GET_CURRENT_JOINT_INFO = "M114"
+    GET_BACK_ZERO_STATUS = "M119"
+    # SET_POINT
 
 class DataProcessor(object):
     # Functional approach
@@ -107,7 +172,7 @@ class DataProcessor(object):
             return sum([list(struct.pack('>h', elem)) for elem in data], [])
 
     def _decode_int8(self, data):
-        return struct.unpack("b", data)[0]
+        return struct.unpack("B", data)[0]
 
     def _decode_int16(self, data):
         return struct.unpack(">h", data)[0]
@@ -146,6 +211,8 @@ class DataProcessor(object):
         return data[pos1] == ProtocolCode.HEADER and data[pos2] == ProtocolCode.HEADER
 
     def _process_received(self, data, genre):
+        if not data:
+            return []
         if genre == 177:
             data = str(data)[2:-1].split(": ")
             return data[1][0:-9], data[-1]
@@ -178,15 +245,24 @@ class DataProcessor(object):
 
         # process valid data
         res = []
-        if data_len == 12 or data_len == 8:
+        if data_len == 12 or data_len == 8 or data_len == 24:
             for header_i in range(0, len(valid_data), 2):
                 one = valid_data[header_i: header_i + 2]
                 res.append(self._decode_int16(one))
         elif data_len == 2:
+            if genre in [ProtocolCode.GET_PLAN_SPEED, ProtocolCode.GET_PLAN_ACCELERATION]:
+                return [self._decode_int8(valid_data[0:1]), self._decode_int8(valid_data[1:])]
             if genre in [ProtocolCode.IS_SERVO_ENABLE]:
                 return [self._decode_int8(valid_data[1:2])]
             res.append(self._decode_int16(valid_data))
+        elif data_len == 3:
+            res.append(self._decode_int16(valid_data[1:]))
         else:
+            if genre in [ProtocolCode.GET_SERVO_VOLTAGES, ProtocolCode.GET_SERVO_STATUS, ProtocolCode.GET_SERVO_TEMPS]:
+                for i in range(data_len):
+                    data1 = self._decode_int8(valid_data[i:i+1])
+                    res.append(0xff & data1 if data1 <0 else data1)
+                return res
             res.append(self._decode_int8(valid_data))
         return res
 
@@ -195,16 +271,16 @@ class DataProcessor(object):
 
 
 def write(self, command, method=None):
+    if len(command)>3 and command[3] == 176 and len(command) > 5:
+        command = "'"+command[4]+"'"+"("+command[5]+")"
+        command = command.encode()
     if method == "socket":
         data = b""
-        if len(command)>3 and command[3] == 176 and len(command) > 5:
-            command = "'"+command[4]+"'"+"("+command[5]+")"
-            command = command.encode()
         if self.rasp:
             self.sock.sendall(str(command).encode())
         else:
             self.sock.sendall(bytes(command))
-        
+
         if len(command) > 3 and command[3] == 177:
             while True:
                 data = self.sock.recv(1024)
@@ -226,14 +302,41 @@ def write(self, command, method=None):
         self.log.debug("_write: {}".format(command))
         self._serial_port.write(command)
         self._serial_port.flush()
-        time.sleep(0.05)
 
-
-def read(self):
-    if self._serial_port.inWaiting() > 0:
-        data = self._serial_port.read(self._serial_port.inWaiting())
-        self.log.debug("_read: {}".format(data))
+def read(self, genre):
+    datas = b""
+    data_len = -1
+    k = 0
+    pre = 0
+    t = time.time()
+    if genre == 0xB1:
+        time.sleep(0.1)
+        if self._serial_port.inWaiting()>0:
+            datas = self._serial_port.read(self._serial_port.inWaiting())
+            print(datas)
+        return datas
+    while True and time.time() - t < 0.1: 
+        data = self._serial_port.read()
+        k+=1
+        if data_len == 1 and data == b'\xfa':
+            datas += data
+            break
+        elif len(datas) == 2:
+            data_len = struct.unpack("b",data)[0]
+            datas += data
+        elif len(datas)>2 and data_len>0:
+            datas += data
+            data_len -= 1
+        elif data == b'\xfe':
+            if datas == b'':
+                datas += data
+                pre = k
+            else:
+                if k-1 == pre:
+                    datas += data
+                else:
+                    datas = b'\xfe'
+                    pre = k  
     else:
-        self.log.debug("_read: no data can be read")
-        data = None
-    return data
+        datas = None
+    return datas
