@@ -14,6 +14,7 @@ from pymycobot.error import calibration_parameters
 
 class MyCobotSocket(MyCobotCommandGenerator):
     """MyCobot Python API Serial communication class.
+    Note: Please use this class under the same network
 
     Supported methods:
 
@@ -48,7 +49,7 @@ class MyCobotSocket(MyCobotCommandGenerator):
     _write = write
     _read = read
 
-    def __init__(self, ip, netport):
+    def __init__(self, ip, netport=9000):
         """
         Args:
             ip: Server ip
@@ -57,10 +58,20 @@ class MyCobotSocket(MyCobotCommandGenerator):
         super(MyCobotSocket, self).__init__()
         self.calibration_parameters = calibration_parameters
         self.SERVER_IP = ip
-        self.SERVER_PORT = 9000
+        self.SERVER_PORT = netport
+        self.rasp = False
         self.sock = self.connect_socket()
 
     def connect(self, serialport="/dev/ttyAMA0", baudrate="1000000", timeout='0.1'):
+        """Connect the robot arm through the serial port and baud rate (Use only when connecting the pi version)
+        
+        Args:
+            serialport: (str) default /dev/ttyAMA0
+            baudrate: default 1000000
+            timeout: default 0.1
+        
+        """
+        self.rasp = True
         self._write(serialport, "socket")
         self._write(baudrate, "socket")
         self._write(timeout, "socket")
@@ -86,9 +97,10 @@ class MyCobotSocket(MyCobotCommandGenerator):
             MyCobotSocket, self)._mesg(genre, *args, **kwargs)
         # [254,...,255]
         data = self._write(self._flatten(real_command), "socket")
-        if data != b'None':
+        if data:
             res = self._process_received(data, genre)
             if genre in [
+                ProtocolCode.ROBOT_VERSION,
                 ProtocolCode.IS_POWER_ON,
                 ProtocolCode.IS_CONTROLLER_CONNECTED,
                 ProtocolCode.IS_PAUSED,
@@ -103,6 +115,7 @@ class MyCobotSocket(MyCobotCommandGenerator):
                 ProtocolCode.GET_SPEED,
                 ProtocolCode.GET_ENCODER,
                 ProtocolCode.GET_BASIC_INPUT,
+                ProtocolCode.GET_TOF_DISTANCE
             ]:
                 return self._process_single(res)
             elif genre in [ProtocolCode.GET_ANGLES]:
@@ -167,9 +180,9 @@ class MyCobotSocket(MyCobotCommandGenerator):
         return self
 
     def set_gpio_mode(self, mode):
-        """Set GPIO mode
+        """Set pin coding method
         Args:
-            mode: BCM or BOARD (str)
+            mode: (str) BCM or BOARD 
         """
         self.calibration_parameters(gpiomode=mode)
         if mode == "BCM":
@@ -178,10 +191,10 @@ class MyCobotSocket(MyCobotCommandGenerator):
             return self._mesg(ProtocolCode.SET_GPIO_MODE, 1)
 
     def set_gpio_out(self, pin_no, mode):
-        """Equivalent to: GPIO.setup(pin_no, mode)
+        """Set the pin as input or output
         Args:
-            pin_no: (int)
-            mode: (str) in --> GPIO.IN  out --> GPIO.OUT 
+            pin_no: (int) pin id
+            mode: (str) "in" or "out"
         """
         if mode == "in":
             return self._mesg(ProtocolCode.SET_GPIO_UP, pin_no, 0)
@@ -189,17 +202,17 @@ class MyCobotSocket(MyCobotCommandGenerator):
             return self._mesg(ProtocolCode.SET_GPIO_UP, pin_no, 1)
 
     def set_gpio_output(self, pin_no, state):
-        """Set state of GPIO pin
+        """Set the pin to high or low level
         Args:
-            pin_no: (int)
-            state: (int) 0 --> GPIO.HIGH  1 --> GPIO.LOW
+            pin_no: (int) pin id.
+            state: (int) 0 or 1
         """
         return self._mesg(ProtocolCode.SET_GPIO_OUTPUT, pin_no, state)
 
     def get_gpio_in(self, pin_no):
-        """Detect state of GPIO pin
+        """Get pin level status.
         Args:
-            pin_no: (int)
+            pin_no: (int) pin id.
         """
         return self._mesg(ProtocolCode.GET_GPIO_IN, pin_no)
 
