@@ -45,9 +45,9 @@ def calibration_parameters(**kwargs):
         degrees = kwargs["degrees"]
         if not isinstance(degrees, list):
             raise MyPalletizedataException("`degrees` must be a list.")
-        if len(degrees) != 4:
+        if len(degrees) not in [3,4]:
             raise MyPalletizedataException(
-                "The length of `degrees` must be 4.")
+                "The length of `degrees` must be 3 /  4.")
         for idx, angle in enumerate(degrees):
             if not MIN_ANGLE <= angle <= MAX_ANGLE:
                 raise MyPalletizedataException(
@@ -97,7 +97,12 @@ class MyPalletizer(MyCobotCommandGenerator):
 
         import serial
 
-        self._serial_port = serial.Serial(port, baudrate, timeout=timeout)
+        self._serial_port = serial.Serial()
+        self._serial_port.port = port
+        self._serial_port.baudrate = baudrate
+        self._serial_port.timeout = timeout
+        self._serial_port.rts = False
+        self._serial_port.open()
 
     _write = write
     _read = read
@@ -120,9 +125,10 @@ class MyPalletizer(MyCobotCommandGenerator):
         self._write(self._flatten(real_command))
 
         if has_reply:
-            data = self._read()
+            data = self._read(genre)
             res = self._process_received(data, genre)
             if genre in [
+                ProtocolCode.ROBOT_VERSION,
                 ProtocolCode.IS_POWER_ON,
                 ProtocolCode.IS_CONTROLLER_CONNECTED,
                 ProtocolCode.IS_PAUSED,
@@ -137,6 +143,7 @@ class MyPalletizer(MyCobotCommandGenerator):
                 ProtocolCode.GET_SPEED,
                 ProtocolCode.GET_ENCODER,
                 ProtocolCode.GET_BASIC_INPUT,
+                ProtocolCode.GET_TOF_DISTANCE
             ]:
                 return self._process_single(res)
             elif genre in [ProtocolCode.GET_ANGLES]:
@@ -146,7 +153,8 @@ class MyPalletizer(MyCobotCommandGenerator):
                     r = []
                     for idx in range(3):
                         r.append(self._int2coord(res[idx]))
-                    r.append(self._int2angle(res[3]))
+                    if len(res)>3:
+                        r.append(self._int2angle(res[3]))
                     return r
                 else:
                     return res
